import asyncio
import os
import pathlib
import sys
import time

from easysmart.manager import Manager
import zeroconf
import paho

from easysmart.mdns.mdns_async import mdns_async_register
from easysmart.mqtt_server.mqtt_server import start_emqx_server
from easysmart.web.main import WebServer


async def test_publish(manager):
    while True:
        await asyncio.sleep(5)
        print('publish')
        await manager.publish('/test/', 'hello world', 0)


def start_server(root_path=None, block=True):
    if sys.platform.lower() == "win32" or os.name.lower() == "nt":
        from asyncio import set_event_loop_policy, WindowsSelectorEventLoopPolicy
        set_event_loop_policy(WindowsSelectorEventLoopPolicy())
    if root_path is None:
        # get the root path of this project
        root_path = pathlib.Path(__file__).parent.parent.absolute()
    print(f'root path is {root_path}')
    # # start the manager
    # asyncio.gather(start_emqx_server(root_path))
    try:
        loop = asyncio.get_event_loop()
    except:
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
    # loop.run_until_complete(mdns_async_register())
    # manager = Manager()
    # asyncio.gather(manager.async_loop_start())
    # web_manager = WebServer(manager)
    # asyncio.gather(web_manager.web_start())
    # # asyncio.gather(test_publish(manager))

    main_manager = Manager()
    asyncio.gather(async_start_server(root_path, main_manager))
    if block:
        try:
            loop.run_forever()
        except KeyboardInterrupt:
            print("stop server")
    else:
        return main_manager, loop
    # asyncio.run(main())

async def async_start_server(root_path=None, main_manager=None):
    if root_path is None:
        # get the root path of this project
        root_path = pathlib.Path(__file__).parent.parent.absolute()
    print(f'root path is {root_path}')
    # start the manager
    asyncio.gather(start_emqx_server(root_path))
    await asyncio.gather(mdns_async_register())
    if manager is None:
        main_manager = Manager()
    asyncio.gather(main_manager.async_loop_start())
    web_manager = WebServer(main_manager)
    asyncio.gather(web_manager.web_start())



import logging

DEVICES_DEFINE = {}

DEVICES_DEFINE['base_device'] = {
    "parent": "",
    "properties": {
        "device_type": {
            "value": "base_device",
            "writable": 0
        }
    },
    "actions": {

    }
}

DEVICES_DEFINE['DIANJI'] = {
    "parent": "base_device",
    "properties": {
        "power": {
            "type": "int",
            "name": "当前电击强度"
        },
        "delay": {
            "type": "int",
            "min": 1,
            "max": 100000,
            "name": "电击间隔ms"
        }
    },
    "actions": {
    },
    "name": "电击器"
}

DEVICES_DEFINE['QTZ01'] = {
    "parent": "base_device",
    "properties": {
        "distance": {
            "type": "float",
            "name": "当前距离",
            "writable": False
        },
        "report_delay_ms": {
            "type": "int",
            "min": 1,
            "max": 100000,
            "name": "汇报间隔"
        }
    },
    "actions": {
    },
    "name": "擎天柱1号"
}

DEVICES_DEFINE['TD01'] = {
    "parent": "base_device",
    "properties": {
        "power1": {
            "type": "int",
            "min": 0,
            "max": 255,
            "name": "1号跳蛋强度"
        },
        "power2": {
            "type": "int",
            "min": 0,
            "max": 255,
            "name": "2号跳蛋强度"
        }
    },
    "actions": {
    },
    "name": "跳弹01"
}


def get_device_define(device_type):
    defines = []
    device_define = _get_device_define(device_type)
    defines.append(device_define)
    while device_define and device_define.get('parent'):
        device_define = _get_device_define(device_define['parent'])
        defines.append(device_define)
    defines.reverse()
    for define in defines:
        if define is None: continue
        device_define['properties'].update(define['properties'])
        device_define['actions'].update(define['actions'])
        for k, v in define.items():
            if k in ['properties', 'actions']: continue
            device_define[k] = v
    return device_define


def _get_device_define(device_type):
    logging.info(f'get device define {device_type}')
    return DEVICES_DEFINE.get(device_type)


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO)
    get_device_define('QTZ01')
    # get_device_define('base_device')



import json
import time

from easysmart.device.device_define import get_device_define
from easysmart.utils import on_log, on_connect, on_disconnect
from paho.mqtt.client import MQTT_LOG_INFO, MQTT_LOG_NOTICE, MQTT_LOG_WARNING, MQTT_LOG_ERR, MQTT_LOG_DEBUG
from paho.mqtt import client as mqtt

class VirtualDevice:
    device_type = 'virtual_device'

    def __init__(self, mac):
        self.power = 0
        self.mac = mac
        assert len(mac) == 12
        device_define = get_device_define(self.device_type)
        if device_define is None:
            raise Exception(f'get_device_define error: {self.device_type}')
        self.properties = device_define['properties']
        self.actions = device_define['actions']
        for property_name in self.properties:
            setattr(self, property_name, self.properties[property_name]['value'])
        self.publish_topic = f'/dpub/{self.mac}'

    def connect_mqtt(self):
        self.client_id = 'MQTT_DEVICE_' + self.mac
        self.client = mqtt.Client(self.client_id, protocol=mqtt.MQTTv311, clean_session=False)
        self.client.on_log = on_log
        self.client.on_connect = on_connect
        self.client.on_message = self.on_message
        self.client.on_disconnect = on_disconnect
        while True:
            try:
                self.client.connect('mqttserver.local', 1883, 60)
                break
            except:
                print('mqtt connect error, retry...')
                time.sleep(1)
        self.client.loop_start()

        self.client.subscribe(f'/drecv/{self.mac}', 1)
        self.client.subscribe('/all', 1)

    def run(self):
        self.connect_mqtt()
        while True:
            self.report()
            time.sleep(10)


    def publish(self, payload, qos=0, retain=False, properties=None):
        return self.client.publish(self.publish_topic, payload, qos, retain, properties)

    def report(self):
        data = {}
        for property_name in self.properties:
            data[property_name] = getattr(self, property_name)
        self.publish(json.dumps(data))

    def update_property(self, property_name, msg_id=-1):
        '''
        example:
        {
            "method":	"update",
            "msg_id":	1,
            "key":	"power2",
            "value":	0
        }
        '''
        data = {
            'method': 'update',
            'msg_id': msg_id,
            'key': property_name,
            'value': getattr(self, property_name) if hasattr(self, property_name) else None,
        }
        self.publish(json.dumps(data))


    def on_message(self, client, userdata, msg):
        print('topic:' + msg.topic + ' ' + str(msg.payload))
        try:
            data = json.loads(msg.payload)
        except:
            data = str(msg.payload)
            print(f'json.loads error: {data}')
            return
        print(f'{data}')
        try:
            self.msg_process(msg, data)
        except Exception as e:
            print(f'on_message error: {e}\n{msg}\n{data}')

    def msg_process(self, msg, data):
        msg_id = data.get('msg_id', -1)
        method = data.get('method', '')
        if method == 'set':
            '''
            example:
            {
              "method": "set",
              "key":"power2",
              "value":0,
              "msg_id":1
            }
            '''
            key = data.get('key', '')
            if key not in self.properties:
                print(f'key not in properties: {key}')
                return
            if not self.properties[key].get('writable', True):
                self.publish_error(f'key not writable: {key}', msg_id)
                return
            value = data.get('value', '')
            setattr(self, key, value)
            self.update_property(key, msg_id)
        elif method == 'get':
            '''
            example:
            {
              "method": "get",
              "key":"power2",
              "msg_id":1
            }
            '''
            key = data.get('key', '')
            self.update_property(key, msg_id)

    def publish_error(self, msg, msg_id):
        data = {
            'method': 'error',
            'msg_id': msg_id,
            'msg': msg,
        }
        self.publish(json.dumps(data))


if __name__ == '__main__':
    device = VirtualDevice('123456789012')
    device.run()



#!/usr/bin/env python3
"""Example of announcing 250 services (in this case, a fake HTTP server)."""

import argparse
import asyncio
import logging
import socket
from typing import List, Optional

from zeroconf import IPVersion
from zeroconf.asyncio import AsyncServiceInfo, AsyncZeroconf


def get_host_ip():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        s.connect(('8.8.8.8', 80))  # 114.114.114.114也是dns地址
        ip = s.getsockname()[0]
    finally:
        s.close()
    return ip


async def mdns_async_register():
    ip_version = IPVersion.V4Only
    desc = {'path': '/~paulsm/'}

    info = AsyncServiceInfo(
        "_http._tcp.local.",
        "EasySmartSever._http._tcp.local.",
        addresses=[socket.inet_aton(get_host_ip())],
        port=80,
        properties=desc,
        server="easysmart.local.",
    )
    runner = AsyncMdnsRunner(IPVersion.V4Only)
    await runner.register_services([info])
    # while True:
    #     await asyncio.sleep(1)


class AsyncMdnsRunner:
    def __init__(self, ip_version: IPVersion) -> None:
        self.ip_version = ip_version
        self.aiozc: Optional[AsyncZeroconf] = None

    async def register_services(self, infos: List[AsyncServiceInfo]) -> None:
        self.aiozc = AsyncZeroconf(ip_version=self.ip_version)
        tasks = [self.aiozc.async_register_service(info) for info in infos]
        background_tasks = await asyncio.gather(*tasks)
        await asyncio.gather(*background_tasks)
        print("Finished registration, press Ctrl-C to exit...")
        # while True:
        #     await asyncio.sleep(1)

    async def unregister_services(self, infos: List[AsyncServiceInfo]) -> None:
        assert self.aiozc is not None
        tasks = [self.aiozc.async_unregister_service(info) for info in infos]
        background_tasks = await asyncio.gather(*tasks)
        await asyncio.gather(*background_tasks)
        await self.aiozc.async_close()


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)

    parser = argparse.ArgumentParser()
    parser.add_argument('--debug', action='store_true')
    version_group = parser.add_mutually_exclusive_group()
    version_group.add_argument('--v6', action='store_true')
    version_group.add_argument('--v6-only', action='store_true')
    args = parser.parse_args()

    if args.debug:
        logging.getLogger('zeroconf').setLevel(logging.DEBUG)
    if args.v6:
        ip_version = IPVersion.All
    elif args.v6_only:
        ip_version = IPVersion.V6Only
    else:
        ip_version = IPVersion.V4Only

    infos = []
    for i in range(250):
        infos.append(
            AsyncServiceInfo(
                "_http._tcp.local.",
                f"Paul's Test Web Site {i}._http._tcp.local.",
                addresses=[socket.inet_aton("127.0.0.1")],
                port=80,
                properties={'path': '/~paulsm/'},
                server=f"zcdemohost-{i}.local.",
            )
        )

    print("Registration of 250 services...")
    loop = asyncio.get_event_loop()
    runner = AsyncMdnsRunner(ip_version)
    try:
        loop.run_until_complete(runner.register_services(infos))
    except KeyboardInterrupt:
        loop.run_until_complete(runner.unregister_services(infos))



import json
import logging
import time

from .device_define import get_device_define


class BaseDevice:

    def __init__(self, mac, device_type, mqtt_client):
        self.device_type = device_type
        self.mac = mac
        self.last_active_time = time.time()
        self.msg_id = 0
        self.mqtt_client = mqtt_client

        self.publish_topic = f'/drecv/{mac}'

        device_define = get_device_define(device_type)

        if device_define is None:
            # device_define = get_device_define('base_device')
            logging.warning(f'get_device_define error: {device_type}')
            # raise Exception(f'get_device_define error: {device_type}')
            self.properties = {}
            self.actions = {}
            self.name = device_type
        else:
            self.properties = device_define['properties']
            self.actions = device_define['actions']
            self.name = device_define['name']

    async def publish(self, data):
        data['msg_id'] = self.msg_id
        self.msg_id += 1
        payload = json.dumps(data)
        await self.mqtt_client.publish(self.publish_topic, payload=payload)
        # self.mqtt_client.publish(self.publish_topic, payload)

    async def get_property(self, property_name):
        data = {
            'method': 'get',
            'key': property_name,
        }
        await self.publish(data)

    async def set_property(self, property_name, value):
        if property_name not in self.properties:
            logging.warning(f'property {property_name} not in {self.mac}, 请自己注意输入数据格式')
        else:
            if self.properties[property_name]['type'] == 'int':
                value = int(value)
            elif self.properties[property_name]['type'] == 'float':
                value = float(value)
        data = {
            'method': 'set',
            'key': property_name,
            'value': value,
        }
        await self.publish(data)

    def execute_action(self, action_name):
        ...

    def update(self, property_name, value):
        update_flag = False
        if property_name not in self.properties:
            logging.warning(f'property {property_name} not in {self.mac}')
        if not hasattr(self, property_name):
            # 如果没有这个属性，就创建一个
            update_flag = True
            setattr(self, property_name, value)
            self.properties[property_name] = {
                'type': 'unknown',
                'name': property_name,
            }
        else:
            # 如果有这个属性，就更新这个属性
            old_value = getattr(self, property_name)
            if old_value != value:
                update_flag = True
                setattr(self, property_name, value)
        if update_flag:
            print(f'update {self.mac}:{property_name}->{value}')
        self.last_active_time = time.time()

    def to_dict(self):
        return {
            'device_type': self.device_type,
            'mac': self.mac,
            'last_active_time': self.last_active_time,
            'properties': {k: getattr(self, k) for k in self.properties.keys()},
        }
